import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Random;

public final class SAPI {
	public SAPI() {
		if (gotPAPIaccess) throw new RuntimeException("Already created a SAPI instance");
		gotPAPIaccess = true;
	}
	
	/*
	 * Harvest Intercept API
	 */
	
	private static ArrayList<IInterceptHarvest> harvestIntercepts = new ArrayList<IInterceptHarvest>();
	public static void interceptAdd(IInterceptHarvest intercept) {
		harvestIntercepts.add(intercept);
	}
	public static boolean interceptHarvest(fd world, gs player, Loc loc, int blockID, int meta) {
		for (IInterceptHarvest inter : harvestIntercepts) {
			if (inter.canIntercept(world,player,loc,blockID,meta)) {
				inter.intercept(world,player,loc,blockID,meta);
				return true;
			}
		}
		return false;
	}
	public static void drop(fd world, Loc loc, iz stack) {
		if (world.B) return;
		
		for (int i2 = 0; i2 < stack.a; i2++) {
			float f1 = 0.7F;
			double d1 = world.r.nextFloat() * f1 + (1.0F - f1) * 0.5D;
			double d2 = world.r.nextFloat() * f1 + (1.0F - f1) * 0.5D;
			double d3 = world.r.nextFloat() * f1 + (1.0F - f1) * 0.5D;
			hl localhj = new hl(world, loc.x() + d1, loc.y() + d2, loc.z() + d3, new iz(stack.c,1,stack.i()));
			localhj.c = 10;
			world.b(localhj);
		}
	}
	
	/*
	 * BlockSet Intercept API
	 */
	
	private static ArrayList<IInterceptBlockSet> setIntercepts = new ArrayList<IInterceptBlockSet>();
	public static void interceptAdd(IInterceptBlockSet intercept) {
		setIntercepts.add(intercept);
	}
	public static int interceptBlockSet(fd world, Loc loc, int blockID) {
		for (IInterceptBlockSet inter : setIntercepts) {
			if (inter.canIntercept(world,loc,blockID)) {
				return inter.intercept(world,loc,blockID);
			}
		}
		return blockID;
	}
	
	/*
	 * Reach API
	 */
	
	private static ArrayList<IReachBlock> reachesBlock = new ArrayList<IReachBlock>();
	private static ArrayList<IReachEntity> reachesEntity = new ArrayList<IReachEntity>();
	public static void reachAdd(IReachBlock reach) {
		reachesBlock.add(reach);
	}
	public static void reachAdd(IReachEntity reach) {
		reachesEntity.add(reach);
	}
	public static float reachGetBlock() {
		iz inHand = ModLoader.getMinecraftInstance().h.c.b();
		for (IReachBlock reach : reachesBlock) if (reach.reachBlockItemMatches(inHand)) return reach.getReachBlock(inHand);
		return 4F;
	}
	public static float reachGetEntity() {
		iz inHand = ModLoader.getMinecraftInstance().h.c.b();
		for (IReachEntity reach : reachesEntity) if (reach.reachEntityItemMatches(inHand)) return reach.getReachEntity(inHand);
		return 3F;
	}
	
	/*
	 * Dungeon API
	 */
	
	private static ArrayList<String> dngMobs = new ArrayList<String>();
	private static ArrayList<DungeonLoot> dngItems = new ArrayList<DungeonLoot>();
	private static ArrayList<DungeonLoot> dngGuaranteed = new ArrayList<DungeonLoot>();
	private static boolean dngAddedMobs = false, dngAddedItems = false;
	
	public static void dungeonAddMob(String mob) {
		dungeonAddMob(mob,10);
	}
	public static void dungeonAddMob(String mob, int chances) {
		for (int i = 0; i < chances; i++) dngMobs.add(mob);
	}
	public static void dungeonRemoveMob(String mob) {
		for (int i = 0; i < dngMobs.size(); i++) {
			if (dngMobs.get(i).equals(mob)) {
				dngMobs.remove(i);
				i--;
			}
		}
	}
	public static void dungeonRemoveAllMobs() {
		dngAddedMobs = true;
		dngMobs.clear();
	}
	static void dungeonAddDefaultMobs() {
		for (int i = 0; i < 10; i++) dngMobs.add("Skeleton");
		for (int i = 0; i < 20; i++) dngMobs.add("Zombie");
		for (int i = 0; i < 10; i++) dngMobs.add("Spider");
	}
	public static String dungeonGetRandomMob() {
		if (!dngAddedMobs) {
			dungeonAddDefaultMobs();
			dngAddedMobs = true;
		}
		if (dngMobs.isEmpty()) return "Pig";
		return dngMobs.get(new Random().nextInt(dngMobs.size()));
	}
	
	public static void dungeonAddItem(DungeonLoot loot) {
		dungeonAddItem(loot,100);
	}
	public static void dungeonAddItem(DungeonLoot loot, int chances) {
		for (int i = 0; i < chances; i++) dngItems.add(loot);
	}
	public static void dungeonAddGuaranteedItem(DungeonLoot loot) {
		dngGuaranteed.add(loot);
	}
	public static int dungeonGetAmountOfGuaranteed() {
		return dngGuaranteed.size();
	}
	public static DungeonLoot dungeonGetGuaranteed(int index) {
		return dngGuaranteed.get(index);
	}
	public static void dungeonRemoveItem(int id) {
		for (int i = 0; i < dngItems.size(); i++) {
			if (dngItems.get(i).loot.c == id) {
				dngItems.remove(i);
				i--;
			}
		}
		for (int i = 0; i < dngGuaranteed.size(); i++) {
			if (dngGuaranteed.get(i).loot.c == id) {
				dngGuaranteed.remove(i);
				i--;
			}
		}
	}
	public static void dungeonRemoveAllItems() {
		dngAddedItems = true;
		dngItems.clear();
		dngGuaranteed.clear();
	}
	static void dungeonAddDefaultItems() {
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.ay)));
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.m),1,4));
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.S)));
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.R),1,4));
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.K),1,4));
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.I),1,4));
		for (int i = 0; i < 100; i++) dngItems.add(new DungeonLoot(new iz(gm.au)));
		dngItems.add(new DungeonLoot(new iz(gm.ar)));
		for (int i = 0; i < 50; i++) dngItems.add(new DungeonLoot(new iz(gm.aA),1,4));
		for (int i = 0; i < 5; i++) dngItems.add(new DungeonLoot(new iz(gm.bd)));
		for (int i = 0; i < 5; i++) dngItems.add(new DungeonLoot(new iz(gm.be)));
	}
	public static iz dungeonGetRandomItem() {
		if (!dngAddedItems) {
			dungeonAddDefaultItems();
			dngAddedItems = true;
		}
		if (dngItems.isEmpty()) return null;
		return dngItems.get(new Random().nextInt(dngItems.size())).getStack();
	}
	
	/*
	 * Achievement Pages
	 */
	
	public static int acCurrentPage = 0;
	private static ArrayList<Integer> acHidden = new ArrayList<Integer>();
	private static ArrayList<ACPage> acPages = new ArrayList<ACPage>();
	public static final ACPage acDefaultPage = new ACPage();
	
	public static void acPageAdd(ACPage page) {
		acPages.add(page);
	}
	public static void acHide(ny... achievements) {
		for (ny achievement : achievements) acHidden.add(achievement.e);
	}
	public static boolean acIsHidden(ny achievement) {
		return acHidden.contains(achievement.e);
	}
	public static ACPage acGetPage(ny achievement) {
		if (achievement == null) return null;
		for (ACPage Page : acPages) if (Page.list.contains(achievement.e)) return Page;
		return acDefaultPage;
	}
	public static ACPage acGetCurrentPage() {
		return acPages.get(acCurrentPage);
	}
	public static String acGetCurrentPageTitle() {
		return acGetCurrentPage().title;
	}
	public static void acPageNext() {
		acCurrentPage++;
		if (acCurrentPage > acPages.size()-1) acCurrentPage = 0;
	}
	public static void acPagePrev() {
		acCurrentPage--;
		if (acCurrentPage < 0) acCurrentPage = acPages.size()-1;
	}
	
	/*
	 * NBT helper
	 */
	
	private static ArrayList<INBT> nbtList = new ArrayList<INBT>();
	public static void addNBT(INBT inbt) {
		nbtList.add(inbt);
	}
	public static ArrayList<INBT> getNBTList() {
		return new ArrayList<INBT>(nbtList);
	}
	
	/*
	 * Difficulty Locker
	 */
	
	private static ArrayList<BaseMod> lockList = new ArrayList<BaseMod>();
	public static void lockDifficulty(BaseMod mod) {
		lockList.add(mod);
	}
	public static String lockedErrorList() {
		String str = "";
		if (lockList.isEmpty()) return str;
		
		for (BaseMod mod : lockList) {
			if (!str.isEmpty()) str += ", ";
			str += mod.toString().substring(4);
		}
		return str;
	}
	public static boolean isLocked(wt handler) {
		if (!mod_SAPI.forceLockingDifficulty) return false;
		return mod_SAPI.lockDifficulty != -1;
	}
	
	/*
	 * PlayerAPI by Flan
	 */
	
	private static boolean gotPAPIaccess = false;
	private static ArrayList<Class<? extends PlayerBase>> playerBaseClasses = new ArrayList<Class<? extends PlayerBase>>();
	public static void PAPIregisterPlayerBase(Class<? extends PlayerBase> pb) {
		playerBaseClasses.add(pb);
	}
	public static PlayerBase PAPIgetPlayerBase(dc player, Class<? extends PlayerBase> pb) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (pb.isInstance(base)) return base;
		return null;
	}
	public ArrayList<PlayerBase> PAPIplayerInit(dc player) {
		ArrayList<PlayerBase> playerBases = new ArrayList<PlayerBase>();
		for (Class<? extends PlayerBase> cls : playerBaseClasses) {
			try {
				Constructor<? extends PlayerBase> c = cls.getDeclaredConstructor(dc.class);
				playerBases.add(c.newInstance(player));
			} catch (Exception e) {e.printStackTrace();}
		}
		return playerBases;
	}
	public boolean PAPIonLivingUpdate(dc player) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		boolean override = false;
		for (PlayerBase base : pSP.playerBases) if (base.onLivingUpdate()) override = true;
		return override;
	}
	public boolean PAPIrespawn(dc player) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.respawn()) override = true;
		return override;
	}
	public boolean PAPImoveFlying(dc player, float x, float y, float z) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.moveFlying(x,y,z)) override = true;
		return override;
	}
	public boolean PAPIupdatePlayerActionState(dc player) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.updatePlayerActionState()) override = true;
		return override;
	}
	public boolean PAPIhandleKeyPress(dc player, int j, boolean flag) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.handleKeyPress(j,flag)) override = true;
		return override;
	}
	public boolean PAPIwriteEntityToNBT(dc player, nu tag) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.writeEntityToNBT(tag)) override = true;
		return override;
	}
	public boolean PAPIreadEntityFromNBT(dc player, nu tag) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.readEntityFromNBT(tag)) override = true;
		return override;
	}
	public boolean PAPIonExitGUI(dc player) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.onExitGUI()) override = true;
		return override;
	}
	public boolean PAPIsetEntityDead(dc player) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.setEntityDead()) override = true;
		return override;
	}
	public boolean PAPIonDeath(dc player, sn killer) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.onDeath(killer)) override = true;
		return override;
	}
	public boolean PAPIattackEntityFrom(dc player, sn attacker, int damage) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.attackEntityFrom(attacker,damage)) override = true;
		return override;
	}
	public double PAPIgetDistanceSq(dc player, double d, double d1, double d2, double answer) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) answer = base.getDistanceSq(d,d1,d2,answer);
		return answer;
	}
	public boolean PAPIisInWater(dc player, boolean inWater) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) inWater = base.isInWater(inWater);
		return inWater;
	}
	public boolean PAPIcanTriggerWalking(dc player, boolean canTrigger) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) canTrigger = base.canTriggerWalking(canTrigger);
		return canTrigger;
	}
	public boolean PAPIheal(dc player, int j) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.heal(j)) override = true;
		return override;
	}
	public int PAPIgetPlayerArmorValue(dc player, int armor) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) armor = base.getPlayerArmorValue(armor);
		return armor;
	}
	public float PAPIgetCurrentPlayerStrVsBlock(dc player, uu block, float f) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) f = base.getCurrentPlayerStrVsBlock(block,f);
		return f;
	}
	public boolean PAPImoveEntity(dc player, double d, double d1, double d2) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.moveEntity(d,d1,d2)) override = true;
		return override;
	}
	public cw PAPIsleepInBedAt(dc player, int x, int y, int z) {
		cw status = null;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) status = base.sleepInBedAt(x,y,z,status);
		return status;
	}
	public float PAPIgetEntityBrightness(dc player, float f, float brightness) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) f = base.getEntityBrightness(f,brightness);
		return f;
	}
	public boolean PAPIpushOutOfBlocks(dc player, double d, double d1, double d2) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.pushOutOfBlocks(d,d1,d2)) override = true;
		return override;
	}
	public boolean PAPIonUpdate(dc player) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.onUpdate()) override = true;
		return override;
	}
	public void PAPIafterUpdate(dc player) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) base.afterUpdate();
	}
	public boolean PAPImoveEntityWithHeading(dc player, float f, float f1) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.moveEntityWithHeading(f,f1)) override = true;
		return override;
	}
	public boolean PAPIisOnLadder(dc player, boolean onLadder) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) onLadder = base.isOnLadder(onLadder);
		return onLadder;
	}
	public boolean PAPIisInsideOfMaterial(dc player, ln material, boolean inMaterial) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) inMaterial = base.isInsideOfMaterial(material,inMaterial);
		return inMaterial;
	}
	public boolean PAPIisSneaking(dc player, boolean sneaking) {
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) sneaking = base.isSneaking(sneaking);
		return sneaking;
	}
	public boolean PAPIdropCurrentItem(dc player) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.dropCurrentItem()) override = true;
		return override;
	}
	public boolean PAPIdropPlayerItem(dc player, iz stack) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.dropPlayerItem(stack)) override = true;
		return override;
	}
	public boolean PAPIdisplayGUIEditSign(dc player, yk sign) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.displayGUIEditSign(sign)) override = true;
		return override;
	}
	public boolean PAPIdisplayGUIChest(dc player, lw inventory) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.displayGUIChest(inventory)) override = true;
		return override;
	}
	public boolean PAPIdisplayWorkbenchGUI(dc player, int x, int y, int z) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.displayWorkbenchGUI(x,y,z)) override = true;
		return override;
	}
	public boolean PAPIdisplayGUIFurnace(dc player, sk furnace) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.displayGUIFurnace(furnace)) override = true;
		return override;
	}
	public boolean PAPIdisplayGUIDispenser(dc player, az dispenser) {
		boolean override = false;
		SAPIEntityPlayerSP pSP = (SAPIEntityPlayerSP)player;
		for (PlayerBase base : pSP.playerBases) if (base.displayGUIDispenser(dispenser)) override = true;
		return override;
	}
}